/*
**   FastIRCd - Internet Relay Chat daemon, src/dynconf.c
**   (C) 1999 Carsten Munk (Techie/Stskeeps) <stskeeps@mp3fans.net>
**
**   Rewritten for FastIRCd
**   Copyright (C) 2000
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define DYNCONF_C
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef _WIN32
#include <utmp.h>
#else
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"

#define DoDebug fprintf(stderr, "[%s] %s | %li\n", babuf, __FILE__, __LINE__);
#define AllocCpy(x,y) if ((x) && type == 1) MyFree((x)); x = (char *) MyMalloc(strlen(y) + 1); strcpy(x,y)
#define XtndCpy(x,y) x = (char *) MyMalloc(strlen(y) + 2); *x = '\0'; strcat(x, "*"); strcpy(x,y)

/* internals */
aConfiguration iConf;
int	 icx = 0;
char	 buf[1024];


/* strips \r and \n's from the line */
void iCstrip(char *line)
{
        char *c;

        if ((c = strchr(line, '\n')))   *c = '\0';
        if ((c = strchr(line, '\r')))   *c = '\0';
}
/* this loads dynamic DCONF */
int	load_conf(char *filename, int type)
{
	FILE	*zConf;
	char    *version = NULL;
	char	*i;

	zConf = fopen(filename, "r");

	if (!zConf)
	{
		if (type == 1)
		{
			sendto_ops("DynConf Fatal Error: Could not load %s !", filename);
			return -1;
		}
			else
		{
			fprintf(stderr, "*              ~~ ERROR ~~\n");
			fprintf(stderr, "* DynConf Fatal Error: Couldn't load %s !!!\n", filename);
			fprintf(stderr, "---------------------------------------------------------------\n");

			exit(-1);
		}
	}
	i = fgets(buf, 1023, zConf);
	if (!i)
	{
		if (type == 1)
		{
			sendto_ops("DynConf Fatal Error: Error reading from %s !", filename);
			return -1;
		}
			else
		{
			fprintf(stderr, "*              ~~ ERROR ~~\n");
			fprintf(stderr, "* DynConf Fatal Error: Couldn't read %s !!!\n", filename);
			fprintf(stderr, "---------------------------------------------------------------\n");
			exit(-1);
		}
	}
	iCstrip(buf);
	version = strtok(buf, "^");
	version = strtok(NULL, "");
		/* is this an ircd.ini file? */
	if (!match("1.*", version))
	{
		/* wrong version */
		if (strcmp(version, DYNCONF_CONF_VERSION))
		{
			if (type == 1)
			{
				sendto_ops("DynConf Fatal Error: %s got a non-compatible version (%s) !", filename, version);
				return -1;
			}
				else
			{
				fprintf(stderr, "*              ~~ ERROR ~~\n");
				fprintf(stderr, "* DynConf Fatal Error: Incompatible version (%s) in %s - please upgrade !!!\n", version, filename);
				fprintf(stderr, "---------------------------------------------------------------\n");
				exit(-1);
			}
		}
		load_conf2(zConf,filename,type);
		return -1;
	}
		else
	if (!match("2.*", version))
	{
		/* network file */
		/* wrong version */
		if (strcmp(version, DYNCONF_NETWORK_VERSION))
		{
			if (type == 1)
			{
				sendto_ops("DynConf Fatal Error: %s got a non-compatible network file version (%s) !", filename, version);
				return -1;
			}
				else
			{
				fprintf(stderr, "*              ~~ ERROR ~~\n");
				fprintf(stderr, "* DynConf Fatal Error: Incompatible network file version (%s) in %s - please upgrade !!!\n", version, filename);
				fprintf(stderr, "---------------------------------------------------------------\n");
				exit(-1);
			}
		}
		load_conf3(zConf, filename, type);
		return -1;
	}
		else
	{
		if (type==1)
		{
			sendto_ops("DynConf Fatal Error: Malformed version header in %s.", filename);
			return -1;
		}
		 else
		{
				fprintf(stderr, "*              ~~ ERROR ~~\n");
				fprintf(stderr, "* DynConf Fatal Error: Malformed or missing version header in %s.\n",filename);
				fprintf(stderr, "---------------------------------------------------------------\n");
				exit(-1);
		}
		return -1;
	}
	return -1;
}

int	load_conf2(FILE *conf, char *filename, int type)
{
	char *databuf = buf;
	char *stat = databuf;
	char *p, *setto, *var;

	if (!conf)
		return -1;

	*databuf = '\0';

	/* loop to read data */
	while (stat != NULL)
	{
		stat = fgets(buf, 1020, conf);
		if ((*buf == '#') || (*buf == '/'))
			continue;

		iCstrip(buf);
		if (*buf == '\0')
			continue;
		p = strtok(buf, " ");
		/*
		** We want this to be case insensitive so we use stricmp instead of strcmp - HAMLET
		*/
		if (stricmp(p, "Include")==0)
		{
			strtok(NULL, " ");
			setto = strtok(NULL, "");
			/* We need this for STATS S -codemastr
			   Isn't there a better way to show all include files? --Stskeeps */
			AllocCpy(INCLUDE, setto);
			load_conf(setto, type);
		}
		/*
		** We want this to be case insensitive so we use stricmp instead of strcmp - HAMLET
		*/
			else
		if (stricmp(p, "Set")==0)
		{
			var = strtok(NULL, " ");
			if (var == NULL)
				continue;
			if (*var == '\0')
				continue;

			strtok(NULL, " ");
			setto = strtok(NULL, "");

			if (strcmp(var, "HALFHUB")==0)
			{
				HALFHUB = atoi(setto);
			}
				else
			if (strcmp(var, "TRUEHUB")==0)
			{
				TRUEHUB = atoi(setto);
			}
				else
			if (strcmp(var, "CLIENT_FLOOD")==0)
			{
				CLIENT_FLOOD = atol(setto);
			 	if (CLIENT_FLOOD > 8000)
				{
					fprintf(stderr, "* DynConf Warning: CLIENT_FLOOD set to large. Resetting it to 8000\n");
					CLIENT_FLOOD = atol("8000");
				}
			 	if (CLIENT_FLOOD < 512)
				{
					fprintf(stderr, "* DynConf Warning: CLIENT_FLOOD set to low. Resetting it to 4000\n");
					CLIENT_FLOOD = atol("4000");
				}
			}
				else
			if (strcmp(var, "geo_location")==0)
			{
				AllocCpy(geo_location, setto);
			}
				else
			if (strcmp(var, "MODE_X")==0)
			{
				MODE_X = atoi(setto);
			}
				else
			if (strcmp(var, "DEF_MODE_X")==0)
			{
				DEF_MODE_X = atoi(setto);
			}
				else
			if (strcmp(var, "DEF_MODE_I")==0)
			{
				DEF_MODE_I = atoi(setto);
			}

				else
			if (strcmp(var, "DEF_MODE_S")==0)
			{
				DEF_MODE_S = atoi(setto);
			}
				else
			if (strcmp(var, "DEF_MODE_W")==0)
			{
				DEF_MODE_W = atoi(setto);
			}
				else
			if (strcmp(var, "SHORT_MOTD")==0)
			{
				SHORT_MOTD = atoi(setto);
			}
				else
			if (strcmp(var, "CONFIG_FILE_STOP")==0)
			{
				if (atoi(setto) == 1)
				{
					fprintf(stderr, "*              ~~ ERROR ~~\n");
					fprintf(stderr, "* DynConf Fatal Error: File stop code recieved in %s!!!\n", filename);
					fprintf(stderr, "* DynConf Fatal Error: You MUST edit your %s file!!!\n", filename);
					fprintf(stderr, "---------------------------------------------------------------\n");
					exit(1);
				}
			}
			else
			{
					fprintf(stderr, "* Dynconf Warning: Invalid %s setting in %s \n", p, filename);

			}

		}
		else
		{

			if (type==0)
			{
					fprintf(stderr, "* Dynconf Warning: Invalid %s setting in %s \n", p, filename);
			}
	 		else
			{
				sendto_realops("Dynconf Warning: Invalid %s setting in %s \n", p, filename);
			}
		}
	}	
	if (type==0)
	{
		fprintf(stderr, " Dynconf Loaded %s\n",filename);
	}
	 else
	{
		sendto_realops("Loaded %s", filename);
	}
}

/* Load .network options */
int	load_conf3(FILE *conf, char *filename, int type)
{
	char *databuf = buf;
	char *stat = databuf;
	char *p, *setto, *var;
	if (!conf)
		return -1;

	*databuf = '\0';
	
	/* loop to read data */
	while (stat != NULL)
	{
		stat = fgets(buf, 1020, conf);
		if ((*buf == '#') || (*buf == '/'))
			continue;
		
		iCstrip(buf);
		if (*buf == '\0')
			continue;

		p = strtok(buf, " ");
		/*
		** We want this to be case insensitive so we use stricmp instead of strcmp - HAMLET
		*/
		if (stricmp(p, "Set")==0)
		{
			var = strtok(NULL, " ");
			if (var == NULL)
				continue;
			if (*var == '\0')
				continue;
				
			strtok(NULL, " ");
			setto = strtok(NULL, "");
			if (strcmp(var, "ircnetwork")==0)
			{
				AllocCpy(ircnetwork, setto);
			} 
				else
			if (strcmp(var, "ircnetwork_name")==0)
			{
				AllocCpy(ircnetwork_name, setto);
			}
				else
			if (strcmp(var, "defserv")==0)
			{
				AllocCpy(defserv, setto);
			}
				else
			if (strcmp(var, "webpage")==0)
			{
				AllocCpy(webpage, setto);
			}
				else
			if (strcmp(var, "webpage_rules")==0)
			{
				AllocCpy(webpage_rules, setto);
			}
				else
			if (strcmp(var, "KLINE_ADDRESS")==0)
			{
				AllocCpy(KLINE_ADDRESS, setto);
			}
				else
			if (strcmp(var, "net_clone_protection")==0)
			{
				net_clone_protection = atoi(setto);
			}
				else
			if (strcmp(var, "net_clone_limit")==0)
			{
				net_clone_limit = atol(setto);
			}
				else
			if (strcmp(var, "net_clone_limit_bypass")==0)
			{
				net_clone_limit_bypass = atoi(setto);
			}
				else
			if (strcmp(var, "HIDE_ULINES")==0)
			{
				HIDE_ULINES = atoi(setto);
			}
				else
			if (strcmp(var, "global_connects")==0)
			{
				GLOBAL_CONNECTS = atoi(setto);
			}
				else
			if (strcmp(var, "network_proxyscan")==0)
			{
				network_proxyscan = atoi(setto);
			}
				else
			if (strcmp(var, "network_proxyscan_notice")==0)
			{
				network_proxyscan_notice = atoi(setto);
			}
				else
			if (strcmp(var, "network_auto_protect")==0)
			{
				network_auto_protect = atoi(setto);
			}
				else
			if (strcmp(var, "net_users_auto_join")==0)
			{
				AllocCpy(net_users_auto_join, setto);
			}

				else
			if (strcmp(var, "net_opers_auto_join")==0)
			{
				AllocCpy(net_opers_auto_join, setto);
			}

				else
			if (strcmp(var, "services_name")==0)
			{
				AllocCpy(services_name, setto);
			}
				else
			if (strcmp(var, "watchdog_name")==0)
			{
				AllocCpy(watchdog_name, setto);
			}
				else
			if (strcmp(var, "statserv_name")==0)
			{
				AllocCpy(statserv_name, setto);
			}
				else
			if (strcmp(var, "locop_host")==0)
			{
				AllocCpy(locop_host, setto);
			}
				else
			if (strcmp(var, "oper_host")==0)
			{
				AllocCpy(oper_host, setto);
			}
				else
			if (strcmp(var, "admin_host")==0)
			{
				AllocCpy(admin_host, setto);
			}
				else
			if (strcmp(var, "servadmin_host")==0)
			{
				AllocCpy(servadmin_host, setto);
			}
				else
			if (strcmp(var, "servicesoper_host")==0)
			{
				AllocCpy(servicesoper_host, setto);
			}
				else
			if (strcmp(var, "servicesadmin_host")==0)
			{
				AllocCpy(servicesadmin_host, setto);
			}
				else
			if (strcmp(var, "servicesroot_host")==0)
			{
				AllocCpy(servicesroot_host, setto);
			}
				else
			if (strcmp(var, "techadmin_host")==0)
			{
				AllocCpy(techadmin_host, setto);
			}
				else
			if (strcmp(var, "netadmin_host")==0)
			{
				AllocCpy(netadmin_host, setto);
			}
				else
			if (strcmp(var, "ircadmin_host")==0)
			{
				AllocCpy(ircadmin_host, setto);
			}
				else
			if (strcmp(var, "sbot_host")==0)
			{
				AllocCpy(sbot_host, setto);
			}
				else
			if (strcmp(var, "netmon_host")==0)
			{
				AllocCpy(netmon_host, setto);
			}
				else
			if (strcmp(var, "hidden_host")==0)
			{
				AllocCpy(hidden_host, setto);
			}
				else
			if (strcmp(var, "helpchan")==0)
			{
				AllocCpy(helpchan, setto);
			}
				else
			if (strcmp(var, "net_enforcer")==0)
			{
				AllocCpy(net_enforcer, setto);
			}
				else
			if (strcmp(var, "net_enforcer_userid")==0)
			{
				AllocCpy(net_enforcer_userid, setto);
			}
				else
			if (strcmp(var, "net_enforcer_host")==0)
			{
				AllocCpy(net_enforcer_host, setto);
			}
				else
			if (strcmp(var, "net_enforcer_realname")==0)
			{
				AllocCpy(net_enforcer_realname, setto);
			}
				else
			if (strcmp(var, "net_enforcer_server")==0)
			{
				AllocCpy(net_enforcer_server, setto);
			}
				else
			if (strcmp(var, "net_enforcer_server_name")==0)
			{
				AllocCpy(net_enforcer_server_name, setto);
			}
			else
			{
					fprintf(stderr, " Dynconf Warning: Invalid %s setting in %s \n", p, INCLUDE);

			}

		}
		else
		{

			if (type==0)
			{
					fprintf(stderr, " Dynconf Warning: Invalid %s setting in %s \n", p, INCLUDE);
			}
	 		else
			{
				sendto_realops("Dynconf Warning: Invalid %s setting in %s \n", p, INCLUDE);
			}
		}
		
	}	
	if (type==0)
	{
		fprintf(stderr, " Dynconf Loaded %s\n",INCLUDE);
	}
	 else
	{
		sendto_realops("Loaded %s", INCLUDE);
	}
}


void	doneconf(int type) {
	/* calculate on errors */
	char	*errormsg = MyMalloc(16384);
	
	*errormsg = '\0';
	if (!KLINE_ADDRESS || (*KLINE_ADDRESS == '\0'))
		strcat(errormsg, "- Missing KLINE_ADDRESS\n");
	if (KLINE_ADDRESS)
		if (!strchr(KLINE_ADDRESS, '@') && !strchr(KLINE_ADDRESS, ':'))
		{
			strcat(errormsg, "- KLINE_ADDRESS is not an e-mail or an URL\n");
		}
#define Missing(x) !(x) || (*(x) == '\0')
	if (Missing(defserv)) strcat(errormsg, "- Missing defserv field\n");
	if (Missing(services_name)) strcat(errormsg, "- Missing services_name field\n");
	if (Missing(oper_host)) strcat(errormsg, "- Missing oper_host field\n");
	if (Missing(admin_host)) strcat(errormsg, "- Missing admin_host field\n");
	if (Missing(locop_host)) strcat(errormsg, "- Missing locop_host field\n");
	if (Missing(netadmin_host)) strcat(errormsg, "- Missing netadmin_host field\n");
	if (Missing(techadmin_host)) strcat(errormsg, "- Missing techadmin_host field\n");
	if (Missing(hidden_host)) strcat(errormsg, "- Missing hidden_host field\n");
	if (Missing(helpchan))  strcat(errormsg, "- Missing helpchan field\n");
	if (Missing(statserv_name)) strcat(errormsg, "- Missing statserv_name field\n");
	if (*errormsg != '\0')
	{
		fprintf(stderr, "\n*** ERRORS IN DYNAMIC CONFIGURATION (listed below) ***\n");
		fprintf(stderr, errormsg);
		MyFree(errormsg);
		fprintf(stderr, "---------------------------------------------------------------\n");
		exit(-1);
	}
	MyFree(errormsg);
}

void init_dynconf(void)
{
	bzero(&iConf, sizeof(iConf));
}

/* Report the unrealircd.conf info -codemastr*/
void report_dynconf (aClient *sptr) 
{
	sendto_one(sptr, ":%s %d %s :\--------------------- Dynconf Settings -------------------\2",
	 		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s :",
      		me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, ":%s %d %s :Include: \2%s\2",
		 me.name, RPL_SETTINGS, sptr->name, INCLUDE);

	sendto_one(sptr, ":%s %d %s :Default Port: \2%d\2",
      		me.name, RPL_SETTINGS, sptr->name, portnum);	

	sendto_one(sptr, ":%s %d %s :Half HUB: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, HALFHUB);	

	sendto_one(sptr, ":%s %d %s :True HUB: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, TRUEHUB);	

	sendto_one(sptr, ":%s %d %s :Flood Level: \2%d\2",
      		me.name, RPL_SETTINGS, sptr->name, CLIENT_FLOOD);	

	sendto_one(sptr, ":%s %d %s :Geo Location: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, geo_location);	

	sendto_one(sptr, ":%s %d %s :Usermode x locked: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, MODE_X);	

	sendto_one(sptr, ":%s %d %s :Usermode x on login: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, DEF_MODE_X);	

	sendto_one(sptr, ":%s %d %s :Usermode i on login: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, DEF_MODE_I);	

	sendto_one(sptr, ":%s %d %s :Usermode s on login: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, DEF_MODE_S);	

	sendto_one(sptr, ":%s %d %s :Usermode w on login: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, DEF_MODE_W);	

	sendto_one(sptr, ":%s %d %s :Short MOTD: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, SHORT_MOTD);	

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s :\2---------------------------------------------------------------\2",
			me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, rpl_str(RPL_ENDOFSETTINGS), me.name, sptr->name);
}

/* Report the network file info -codemastr */
void report_network (aClient *sptr)
{
	sendto_one(sptr, ":%s %d %s :\2---------------------Network Info ------------------------\2",
	 		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);	

	sendto_one(sptr, ":%s %d %s :Network File: \2%s\2",
		 me.name, RPL_SETTINGS, sptr->name, ircnetwork);

	sendto_one(sptr, ":%s %d %s :Network Name: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, ircnetwork_name);

	sendto_one(sptr, ":%s %d %s :Default Server: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, defserv);

	sendto_one(sptr, ":%s %d %s :Network Web Site: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, webpage);

	sendto_one(sptr, ":%s %d %s :Network Rules Page: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, webpage_rules);

	sendto_one(sptr, ":%s %d %s :K-Line Address: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, KLINE_ADDRESS);


	sendto_one(sptr, ":%s %d %s :Network Help Channel: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, helpchan);	

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, ":%s %d %s :\2---------------------------------------------------------------\2",
			me.name, RPL_SETTINGS, sptr->name);


	sendto_one(sptr, rpl_str(RPL_ENDOFSETTINGS), me.name, sptr->name);
}

/* Report Options */
void report_options (aClient *sptr)
{
	sendto_one(sptr, ":%s %d %s :\2----------------------- Network Options ----------------------\2",
	 		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);	

	sendto_one(sptr, ":%s %d %s :Clone Protection: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, net_clone_protection);

	sendto_one(sptr, ":%s %d %s :Allowed Connections Per Host: \2%d\2",
      		me.name, RPL_SETTINGS, sptr->name, net_clone_limit);

	sendto_one(sptr, ":%s %d %s :Oper Bypass Connection Limit: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, net_clone_limit_bypass);

	sendto_one(sptr, ":%s %d %s :Hide U Lines: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, HIDE_ULINES);

	sendto_one(sptr, ":%s %d %s :Show Global Connect Notices: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, GLOBAL_CONNECTS);

	sendto_one(sptr, ":%s %d %s :Scan For Proxy Servers: \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, network_proxyscan);

	sendto_one(sptr, ":%s %d %s :Notify On Proxy Scan Excempts (e): \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, network_proxyscan_notice);

	sendto_one(sptr, ":%s %d %s :Auto Set +p (Protected Oper): \2%i\2",
      		me.name, RPL_SETTINGS, sptr->name, network_auto_protect);

	sendto_one(sptr, ":%s %d %s :Users Auto Join Channel(s): \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, net_users_auto_join);	

	sendto_one(sptr, ":%s %d %s :Opers Auto Join Channel(s): \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, net_opers_auto_join);	

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);


	sendto_one(sptr, ":%s %d %s :\2---------------------------------------------------------------\2",
			me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, rpl_str(RPL_ENDOFSETTINGS), me.name, sptr->name);
}


/* Report Hostmasks */
void report_hostmasks (aClient *sptr)
{
	sendto_one(sptr, ":%s %d %s :\2----------------------- Network Hostmasks ----------------------\2",
	 		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s :Hidden Host Prefix: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, hidden_host);

	sendto_one(sptr, ":%s %d %s :Local Operator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, locop_host);

	sendto_one(sptr, ":%s %d %s :Global Operator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, oper_host);

	sendto_one(sptr, ":%s %d %s :Administrator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, admin_host);

	sendto_one(sptr, ":%s %d %s :Server Administrator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, servadmin_host);

	sendto_one(sptr, ":%s %d %s :Services Oper Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, servicesoper_host);

	sendto_one(sptr, ":%s %d %s :Services Administrator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, servicesadmin_host);	

	sendto_one(sptr, ":%s %d %s :Services Root Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, servicesroot_host);	

	sendto_one(sptr, ":%s %d %s :Technical Administrator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, techadmin_host);	

	sendto_one(sptr, ":%s %d %s :Network Administrator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, netadmin_host);	

	sendto_one(sptr, ":%s %d %s :IRC Administrator Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, ircadmin_host);	

	sendto_one(sptr, ":%s %d %s :Server Bot Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, sbot_host);	

	sendto_one(sptr, ":%s %d %s :Network Monitor Bot Hostname: %s",
      		me.name, RPL_SETTINGS, sptr->name, netmon_host);	

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, ":%s %d %s :\2---------------------------------------------------------------\2",
			me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, rpl_str(RPL_ENDOFSETTINGS), me.name, sptr->name);
}

/* Report services */
void report_services (aClient *sptr)
{
	sendto_one(sptr, ":%s %d %s :\2---------------------- Services Settings -----------------------\2",
	 		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s :Services Server: %s",
      		me.name, RPL_SETTINGS, sptr->name, services_name);

	sendto_one(sptr, ":%s %d %s :WatchDog Server: %s",
      		me.name, RPL_SETTINGS, sptr->name, watchdog_name);

	sendto_one(sptr, ":%s %d %s :StatServ Server: %s",
      		me.name, RPL_SETTINGS, sptr->name, statserv_name);

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, ":%s %d %s :\2---------------------------------------------------------------\2",
			me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, rpl_str(RPL_ENDOFSETTINGS), me.name, sptr->name);
}



/* Report Misc */
void report_misc (aClient *sptr)
{
	sendto_one(sptr, ":%s %d %s :\2---------------------- Misc Network Settings ------------------\2",
	 		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);

	sendto_one(sptr, ":%s %d %s :Network Enforcer: \2%s (%s@%s)\2",
      		me.name, RPL_SETTINGS, sptr->name, net_enforcer, net_enforcer_userid, net_enforcer_host);	

	sendto_one(sptr, ":%s %d %s :Network Enforcer Realname: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, net_enforcer_realname);	

	sendto_one(sptr, ":%s %d %s :Network Enforcer Server: \2%s\2 ",
      		me.name, RPL_SETTINGS, sptr->name, net_enforcer_server);

	sendto_one(sptr, ":%s %d %s :Network Enforcer Server Realname: \2%s\2",
      		me.name, RPL_SETTINGS, sptr->name, net_enforcer_server_name);	

	sendto_one(sptr, ":%s %d %s : ",
      		me.name, RPL_SETTINGS, sptr->name);	


	sendto_one(sptr, ":%s %d %s :\2---------------------------------------------------------------\2",
			me.name, RPL_SETTINGS, sptr->name);


	sendto_one(sptr, rpl_str(RPL_ENDOFSETTINGS), me.name, sptr->name);
}
