/*
**   FastIRCd - Internet Relay Chat Daemon, src/s_gline.c
**   Copyright (C) 1999 Carsten Munk (Techie/Stskeeps) <stskeeps@tspre.org>
**   
**	 FastIRCd
**   Copyright (C) 2000
**
**   This program is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 1, or (at your option)
**   any later version.
**
**   This program is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with this program; if not, write to the Free Software
**   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "userload.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef _WIN32
#include <utmp.h>
#else
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"
#include "version.h"

struct dhms_s {
        int d,h,m,s;
};

char   	*seconds_to_dhms (int secs);
char   	*seconds_to_dhms2 (int secs);


aGline   *expire_gline (aGline *tmp);
void    check_expire_gline (void);


aGline	*glines = NULL;


char *seconds_to_dhms (secs)
	int secs;
{
	struct dhms_s time;
	static char dhms[20 * 4];
	int 	tmp_days, tmp_hours, tmp_minutes, tmp_seconds;
	char d[20], h[20], m[20], s[20];

	tmp_days = tmp_hours = tmp_minutes = tmp_seconds = 0;

	if (secs >= 86400) {
	tmp_days = secs/86400;
	}
	if (secs >= 3600) {
	tmp_hours = (secs/3600)%24;
	}
	if (secs >= 60) {
	tmp_minutes = (secs/60)%60;
	}
	tmp_seconds = secs%60;

	ircsprintf (d, "%d day%s,", tmp_days, tmp_days>1 ? "s" : "");
	ircsprintf (h, " %d hour%s,", tmp_hours, tmp_hours>1 ? "s" : "");
	ircsprintf (m, " %d minute%s,", tmp_minutes, tmp_minutes>1 ? "s" : "");
	ircsprintf (s, " %d second%s", tmp_seconds, tmp_seconds>1 ? "s" : "");

		strcpy	(dhms, "");

	if (tmp_days)
		strcat (dhms, d);
	if (tmp_hours)
		strcat (dhms, h);
	if (tmp_minutes)
		strcat (dhms, m);
	if (tmp_seconds)
		strcat (dhms, s);


	return dhms;
} 

char *seconds_to_dhms2 (secs)
	int secs;
{
	struct dhms_s time;
	static char dhms[20 * 4];
	int 	tmp_days, tmp_hours, tmp_minutes, tmp_seconds;
	char d[20], h[20], m[20], s[20];

	tmp_days = tmp_hours = tmp_minutes = tmp_seconds = 0;

	if (secs >= 86400) {
	tmp_days = secs/86400;
	}
	if (secs >= 3600) {
	tmp_hours = (secs/3600)%24;
	}
	if (secs >= 60) {
	tmp_minutes = (secs/60)%60;
	}
	tmp_seconds = secs%60;

	ircsprintf (d, "%d day%s,", tmp_days, tmp_days>1 ? "s" : "");
	ircsprintf (h, " %d hour%s,", tmp_hours, tmp_hours>1 ? "s" : "");
	ircsprintf (m, " %d minute%s,", tmp_minutes, tmp_minutes>1 ? "s" : "");
	ircsprintf (s, " %d second%s", tmp_seconds, tmp_seconds>1 ? "s" : "");

		strcpy	(dhms, "");

	if (tmp_days)
		strcat (dhms, d);
	if (tmp_hours)
		strcat (dhms, h);
	if (tmp_minutes)
		strcat (dhms, m);
	if (tmp_seconds)
		strcat (dhms, s);


	return dhms;
} 



/* 
   init_glines
*/

void init_glines(void) {
}

#define AllocCpy(x,y) x = (char *) MyMalloc(strlen(y) + 1); strcpy(x,y)
#define GFreeStr(x) MyFree((char *) x)
#define GFreeGline(x) MyFree((aGline *) x)
/*
 * add_gline
 *   adds a gline to the linked list
 *  coded by Stskeeps
*/

void add_gline(usermask, hostmask, reason,setby, expire_at, set_at) 
char *usermask, *hostmask, *reason, *setby;
time_t expire_at, set_at;
{
	aGline *ng;
     
    /* acknowledged - no bugs */	
	ng = (aGline *) MyMalloc(sizeof(aGline));
	
	AllocCpy(ng->usermask, usermask);
	AllocCpy(ng->hostmask, hostmask);
	AllocCpy(ng->reason,   reason);
	AllocCpy(ng->setby,    setby);
	
	ng->expire_at 		 = expire_at;
	ng->set_at		= set_at;
	ng->prev		= NULL;
	ng->next		= glines;
	if (glines)
		glines->prev = ng;	
	glines = ng;
}


/* 
 * del_gline 
 *   coded & recoded by Stskeeps
 *   return pointer to next if found
 *   NULL = not found
 *   
*/
aGline *del_gline(glinex)
aGline *glinex;
{	
	aGline *p, *q;
	
	for (p = glines; p; p = p->next) {
		if (p == glinex) {
			q = p->next;
			GFreeStr(p->hostmask);
			GFreeStr(p->usermask);
			GFreeStr(p->reason);
			GFreeStr(p->setby);	
			/* chain1 to chain3 */
			if (p->prev) {
				p->prev->next = p->next;
			}
			  else
			{
				glines = p->next;
			}
			if (p->next) {
				p->next->prev = p->prev;
			}
			MyFree((aGline *) p);
			return q;			
		}
	}	
	return NULL;
}



/*
	find_gline
	by stskeeps (recoded)
	returns 1 if cptr is glined
*/
int find_gline (cptr)
	aClient *cptr;
{
	char	*chost, *cname;
	aGline	*glp;
	time_t	nowtime;
		
	if (IsServer(cptr) || IsMe(cptr))
		return 0;
	
	check_expire_gline();

	nowtime = TStime();
	chost = cptr->sockhost;
	cname = cptr->user->username;	

	for (glp = glines; glp; glp = glp->next)
	{

		if ((!match(glp->hostmask, chost) && !match(glp->usermask, cname)) || (!match(glp->hostmask, inetntoa((char *) &cptr->ip)) && !match(glp->usermask, cname)))
		{
			sendto_one(cptr, ":%s NOTICE %s :*** Notice -- You are \2G-Lined\2 for \2%s\2 (reason: %s).",
					 me.name, cptr->name,
					 seconds_to_dhms((glp->expire_at - TStime())),
					 glp->reason);
			return 1;
		}
	}
	return 0;
}

void check_expire_gline (void)
{
	aGline *gp, t;
	time_t nowtime;	

	nowtime = TStime();

	for (gp = glines; gp; gp = gp->next)
	{
		if (gp->expire_at <= nowtime)
		{
			t.next = expire_gline(gp);
			gp = &t;
		}
	}
}

aGline *expire_gline (aGline *tmp)
{
	FILE *glinelog;
	
	if (!tmp)
		return NULL;
	
	if (tmp->expire_at > TStime()) {
		sendto_ops("expire_gline(): expire for not-yet-expired gline %s@%s",
			tmp->usermask,
			tmp->hostmask);
		return (tmp->next);	
	}

	sendto_netglobal("from %s : Expiring G:Line (%s@%s) made by %s (Reason: %s)", me.name, tmp->usermask, tmp->hostmask, tmp->setby, tmp->reason);

	glinelog = fopen (GLINE_LOG, "a");
	if (glinelog == NULL) {
     		sendto_realops("Couldn't write to gline.log - Disk full?");
		goto nook1;
	}
	
        fprintf (glinelog, "(%s) - Expiring G:line %s@%s made by %s (Reason: %s)\n",
    		myctime(TStime()), tmp->usermask, tmp->hostmask, 
    		tmp->setby, tmp->reason); 
        fclose (glinelog);
	nook1:
	return(del_gline(tmp));
}

void expire_glines ()
{
/*
 * Stskeeps: As we are using linked lists now this proc is unneeded
 *           will maybe be used later for a /GLINE EXPIREALL ?
 * 
	int	i;
	FILE *glinelog;

        sendto_realops("*** G:Line maximal count reached, expiring"
			" all G:Lines");
	for (i=0 ; i<MAXGLINE ; i++)
	{
		if (glinetab[i] != NULL)
		{
			MyFree (glinetab[i]);
			glinetab[i] = NULL;
		}
	}
	lastgline = 0;

        glinelog = fopen (GLINE_LOG, "a");
		if (glinelog == NULL) {
		 sendto_realops("Couldn't write to gline.log - Disk full?");
		 return;
		}
		fprintf (glinelog, "%s - FATAL: Maximum amount of G:Lines reached -
			all G:lines expiring!\n", myctime(TStime()));
        fclose (glinelog);
*/

}
	
int	gline() {
	/* just sweeps net for people that should be killed */
	aClient *acptr;
		long i,i1;

	check_expire_gline();

/*	for (acptr = client ; acptr ; acptr=acptr->next) {
		if (IsPerson(acptr)) {
			sendto_ops("GLINEDEBUG: %s %s@%s (%s)", acptr->name, 
				acptr->user->username,
				acptr->user->realhost,
				acptr->sockhost);
			if (find_gline(acptr) == 1) {
				exit_client (NULL, acptr, &me, "User has been banned [G:Lined]");
			}
		}
	}
	*/

 for (i1 = 0; i1 <= 5; i1++) 
 {
	for (i = 0; i <= (MAXCONNECTIONS -1) ; i++ )
	{
		if (acptr = local[i])
			if (MyClient(acptr))
				if (find_gline(acptr))
				{
					exit_client (NULL, acptr, &me, "User has been banned [G:Lined]");
				}
	}
  }
	return 1;

	return 1;
}

/*
** dump_gline give list of current glines, times, and reasons
** Recoded by Stskeeps
*/
void dump_gline (aClient *sptr) {
	int	i; time_t curtime;
	int	tmp_left;
	int	tmp_since;

	aGline *p, t;

	curtime = TStime();
	check_expire_gline();
	for (p = glines; p; p = p->next) {

	tmp_left = (p->expire_at - curtime);
	tmp_since = (curtime - p->set_at);

/*
		sendto_one(sptr, rpl_str(RPL_STATSGLINE), me.name, sptr->name,
			p->usermask, p->hostmask, (p->expire_at - curtime),
			(curtime - p->set_at),
			p->setby, p->reason);
*/
		sendto_one(sptr, rpl_str(RPL_STATSGLINE), me.name, sptr->name,
			p->usermask, p->hostmask, seconds_to_dhms(tmp_left),
			seconds_to_dhms2 (tmp_since),
			p->setby, p->reason);


	}
}

/*
** m_remgline
** parv[0] = sender
** parv[1] = hostmask
*/
int m_remgline (aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int	i;
	char	*p;
	aGline 	*gl;
	FILE	*glinelog;

        if (!IsOper (sptr))
        {
                sendto_one(sptr,err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
                return 0;
        }
	
	if (parc < 2)
	{
notenoughparams:
                sendto_one (sptr, err_str(ERR_NEEDMOREPARAMS),
                            me.name, parv[0], "REMGLINE");
                return 0;
        }

	p = strchr (parv[1], '@');
	if (p == NULL) goto notenoughparams;	/* ugly */

	*p = 0;
	check_expire_gline;
	gl = glines;
	while (gl != NULL) {
		if (!match(gl->hostmask, p+1) &&
		    !match(gl->usermask, parv[1]))
		{
			sendto_netglobal("from %s : %s has removed G:Line for %s@%s", me.name, sptr->name, gl->usermask, gl->hostmask);
				
			sendto_serv_butone(cptr, ":%s REMGLINE :%s@%s", sptr->name, gl->usermask, gl->hostmask);

       		        glinelog = fopen (GLINE_LOG, "a");
			if (glinelog == NULL) {
				sendto_realops("Couldn't open gline.log - Disk full?");
				goto nook2;
			}
			fprintf (glinelog, "%s - %s removed G:line for %s@%s\n",
				myctime(time(NULL)), 
				sptr->name,
				gl->usermask,
				gl->hostmask);
      			 fclose (glinelog);
nook2:
	 		del_gline(gl);
			return 0;
		  }
	 	   else
		{
		   	gl = gl->next;	
		}
	}
	
	sendto_one (sptr, ":%s NOTICE %s :*** /RemGLine - No match for hostmask"
			" %s@%s",me.name,sptr->name,parv[1],p+1);
	return 0;
}

/*
** m_gline
** Recoded by Stskeeps
** Recoded SECOND time by Stskeeps
** parv[0] = sender
** parv[1] = nick or hostmask
** parv[2] = time

** parv[3] = reason (if !MyClient its Sender)
** parv[4] = reason (if !MyCLient)
*/
int m_gline (aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	/* client: GLINE mask secs :Reason */
	/* server: :server GLINE mask expire_at set_at sender :Reason */
	char	*sender, *mask, *timestr, *reason, *set_at;
	char	*usermask, *hostmask, *say;
	char	*s, *p;
	char	gmt[256];
	aGline	*gl;
	time_t	curtime, secs, secs2;
	FILE	*glinelog;
	long	i;
	char	tempmask[256];

	if (parc == 1)
	{
		dump_gline(sptr);	
		return 0;
	}

	if (!IsOper(sptr) && !IsServer(sptr))
	{
		if (!IsServer(sptr))
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, sptr->name);
		return 0;
	}

	if (MyOper(sptr)) 
	{
		if (parc < 4)
		{
		sendto_one (sptr, ":%s NOTICE %s :Help on GLINE", me.name, parv[0]);
		sendto_one (sptr, ":%s NOTICE %s :Syntax /GLINE <hostname> <time> :<Reason>", me.name, parv[0]);
		sendto_one (sptr, ":%s NOTICE %s :Time: 3600 = 1 Hour, 86400 = 1 Day, 2592000 = 30 Days.", me.name, parv[0]);
			return;
	
		}
//		sender = s = make_nick_user_host(sptr->name, sptr->user->username, (IsHidden(sptr) ? sptr->user->virthost : sptr->user->realhost));

/*
**
** These notices only go to opers unles i am mistaken.
** I want to see the realhost of the user who set a gline.
** This way opers cannot user /SetHost, /SetName and /SetIdent to make it look
** like another oper set the G Line. - HAMLET
**
*/
		sender = s = make_nick_user_host(sptr->name, sptr->user->username, sptr->user->host);
		mask   = parv[1];
		timestr= parv[2];
		reason = parv[3]; 		
	}
	
	if (IsServer(sptr))
	{
		if (parc < 5)
		{
			sendto_one (sptr, err_str(ERR_NEEDMOREPARAMS),
						me.name, sptr->name, "GLINE");
			return;
		}
		mask 	= parv[1];
		timestr = parv[2];
		set_at  = parv[3];

		sender  = parv[4];
		reason  = parv[5];
	}

	if (strlen(mask) > (HOSTLEN + USERLEN ) ) {
	sendto_one (sptr,":%s NOTICE %s :*** Notice -- AHHHHHH. You trying to kill me? Try a shorter mask.",
				me.name, parv[0]);
		return 0;
	}

	if (mask[0] == '@')
	{
		strcpy(tempmask, "*");
		strcat(tempmask, mask);
		mask = tempmask;
	}

	/* Check if its a hostmask AND legal.. */
	p = strchr(mask, '@');
	if (!p)
	{
		sendto_one (sptr,":%s NOTICE %s :*** Use a hostmask (G:Line)",
				me.name, parv[0]);
		return 0;
	}

	if (p)
	{
		p++;
		i = 0;
		while (*p)
		{
			if (*p != '*' && *p != '.') i++;
			p++;
		}
			if (i<4)
			{
				sendto_netglobal("from %s : Addition of G:Line failed for %s (%s)", me.name, mask, sptr->name);
				return 0;
			}
	}

	/* user@host.. */
	usermask = strtok(mask, "@");
	hostmask = strtok(NULL, "@");

	check_expire_gline();

	gl = glines;
	
	while (gl != NULL) {
		if (!match(gl->hostmask, hostmask) &&
		    !match(gl->usermask, usermask))
		{
			if (!IsServer(sptr)) {
				sendto_one(sptr, ":%s NOTICE %s :*** Cannot add G:Line - Match already exists!", me.name, sptr->name);
			}
			return 0;
		}
		gl = gl->next;
	}

	/* No matches.. */
	curtime = TStime();
	secs = atol(timestr);

	if (IsServer(sptr))
	{
		secs = secs - curtime;
	}		

	if (secs < 1) {
		sendto_one(sptr, ":%s NOTICE %s :*** G:Line - Please specify a POSITIVE value for expiry time", me.name, sptr->name);
		return 0;
	}
/*	
	secs2 = secs;
	if (secs < 60) {
		say = "second(s)";
	}
	if ((secs > 59) && (secs < 3600)) {
		secs2 = secs / 60;
		say = "minute(s)"; 
	}
	if ((secs > 3599) && (secs < 86400)) {
		secs2 = secs / 60 / 60;
		say = "hour(s)";
	}
	if ((secs > 86399) && (secs < 999999999)) {
		secs2 = secs / 60 / 60 / 24;
		say = "day(s)";	
	}
	if (secs > 999999998) {
		sendto_one (sptr, ":%s NOTICE %s :*** Couldn't add G:Line - Seconds value too large - Please decrease", me.name, sptr->name);
		return 0;
	}
*/
	
	/* GMT clock. */
	strncpy (gmt, asctime(gmtime((clock_t*)&curtime)), sizeof(gmt));
	gmt[strlen(gmt)-1] = 0;

	/* add_gline */
	add_gline(usermask, hostmask, reason, sender, curtime + secs, (IsServer(sptr) ? atol(set_at) : curtime));
	
	if (!IsServer(sptr))
	{
//	sendto_netglobal("from %s : G:Line added for \2%s@%s\2 on %s GMT (From %s for approximately %d %s Reason: \2%s\2)", me.name, usermask, hostmask, gmt, sender, secs2, say, reason);
	sendto_netglobal("from %s : G:Line added for \2%s@%s\2 on %s GMT (From %s for %s Reason: \2%s\2)", me.name, usermask, hostmask, gmt, sender, seconds_to_dhms(secs), reason);
	}
	sendto_serv_butone(cptr, ":%s GLINE %s@%s %li %li %s :%s",
		me.name,
		usermask, hostmask,
		(curtime + secs),
		(IsServer(sptr) ? atol(set_at) : curtime),
		sender,
		reason);
	

	/* log it in the gline.log */
	glinelog = fopen (GLINE_LOG, "a");

	if (glinelog == NULL) {
		sendto_realops("*** Couldn't log to gline.log! Disk full?");
		goto nowrite;
	}

	fprintf(glinelog, "(%s) %s@%s G:line added by %s (%s) for %s\n",
		myctime(TStime()),
		usermask,
		hostmask,
		sender,
		reason,
		seconds_to_dhms2(secs));
		

	fclose(glinelog);	

	nowrite:		
	gline();

	return 1;

}    	

